/// <reference types="webgl2" />
import { BufferBits } from "./types";
import { Command } from "./command";
import { Attributes } from "./attributes";
import { Framebuffer } from "./framebuffer";
export interface DeviceMountOptions {
    element?: HTMLElement;
    alpha?: boolean;
    antialias?: boolean;
    depth?: boolean;
    stencil?: boolean;
    preserveDrawingBuffer?: boolean;
    extensions?: Extension[];
    debug?: boolean;
    pixelRatio?: number;
    viewport?: [number, number];
}
export interface DeviceFromCanvasOptions {
    alpha?: boolean;
    antialias?: boolean;
    depth?: boolean;
    stencil?: boolean;
    preserveDrawingBuffer?: boolean;
    extensions?: Extension[];
    debug?: boolean;
    pixelRatio?: number;
    viewport?: [number, number];
}
export interface DeviceFromContextOptions {
    extensions?: Extension[];
    debug?: boolean;
    pixelRatio?: number;
    viewport?: [number, number];
}
export interface TargetClearOptions {
    r?: number;
    g?: number;
    b?: number;
    a?: number;
    depth?: number;
    stencil?: number;
}
/**
 * Available extensions.
 */
export declare enum Extension {
    EXTColorBufferFloat = "EXT_color_buffer_float",
    OESTextureFloatLinear = "OES_texture_float_linear",
}
export declare class Device {
    /**
     * Create a new canvas and device (containing a gl context). Mount it on
     * `element` parameter (default is `document.body`).
     */
    static mount(options?: DeviceMountOptions): Device;
    /**
     * Create a new device (containing a gl context) from existing canvas.
     */
    static fromCanvas(canvas: HTMLCanvasElement, options?: DeviceFromCanvasOptions): Device;
    /**
     * Create a new device from existing gl context.
     */
    static fromContext(gl: WebGL2RenderingContext, {pixelRatio, viewport, extensions, debug}?: DeviceFromContextOptions): Device;
    readonly gl: WebGL2RenderingContext;
    readonly canvas: HTMLCanvasElement;
    private explicitPixelRatio?;
    private explicitViewport?;
    private backbufferTarget;
    private constructor();
    /**
     * Return width of the gl drawing buffer.
     */
    readonly bufferWidth: number;
    /**
     * Return height of the gl drawing buffer.
     */
    readonly bufferHeight: number;
    /**
     * Return width of the canvas. This will usually be the same as:
     *   device.bufferWidth
     */
    readonly canvasWidth: number;
    /**
     * Return height of the canvas. This will usually be the same as:
     *   device.bufferHeight
     */
    readonly canvasHeight: number;
    /**
     * Return width of canvas in CSS pixels (before applying device pixel ratio)
     */
    readonly canvasCSSWidth: number;
    /**
     * Return height of canvas in CSS pixels (before applying device pixel ratio)
     */
    readonly canvasCSSHeight: number;
    /**
     * Return the device pixel ratio for this device
     */
    readonly pixelRatio: number;
    /**
     * Notify the device to check whether updates are needed. This resizes the
     * canvas, if the device pixel ratio or css canvas width/height changed.
     */
    update(): void;
    /**
     * Request a render target from the device to draw into. This gives you the
     * gl.BACK target.
     *
     * Drawing should be done within the callback by
     * calling `ratget.clear()` or `target.draw()` family of methods.
     *
     * Also see `framebuffer.target()`.
     */
    target(cb: (rt: Target) => void): void;
}
/**
 * Target represents a drawable surface. Get hold of targets with
 * `device.target()` or `framebuffer.target()`.
 *
 * Targets provide methods for drawing to a surface and clearing its values.
 */
export declare class Target {
    private gl;
    readonly glDrawBuffers: number[];
    readonly glFramebuffer: WebGLFramebuffer | undefined;
    readonly width: number | undefined;
    readonly height: number | undefined;
    constructor(gl: WebGL2RenderingContext, glDrawBuffers: number[], glFramebuffer?: WebGLFramebuffer | undefined, width?: number | undefined, height?: number | undefined);
    /**
     * Run the callback with the target bound. This is called automatically,
     * when obtaining a target via `device.target()` or `framebuffer.target()`.
     *
     * All drawing to the target should be done within the callback. Otherwise
     * the behavior is undefined.
     */
    with(cb: (rt: Target) => void): void;
    /**
     * Blit source framebuffer onto this render target. Use buffer bits to
     * choose, which buffers to blit.
     */
    blit(source: Framebuffer, bits: BufferBits): void;
    /**
     * Clear selected buffers to provided values.
     */
    clear(bits: BufferBits, {r, g, b, a, depth, stencil}?: TargetClearOptions): void;
    /**
     * Draw to this target with a command, attributes, and command properties.
     * The properties are passed to the command's uniform or texture callbacks,
     * if used.
     */
    draw<P>(cmd: Command<P>, attrs: Attributes, props: P): void;
    /**
     * Perform multiple draws to this target with the same command, but multiple
     * attributes and command properties. The properties are passed to the
     * command's uniform or texture callbacks, if used.
     */
    batch<P>(cmd: Command<P>, cb: (draw: (attrs: Attributes, props: P) => void) => void): void;
    private drawArrays(primitive, count, offset, instanceCount);
    private drawElements(primitive, count, type, offset, instCount);
    private depth(depthDescr?);
    private stencil(stencilDescr?);
    private blend(blendDescr?);
    private textures<P>(textureAccessors, props, index);
    private uniforms<P>(uniformDescrs, props, index);
}
